"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebsocketClient = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const constants_1 = require("./constants");
const message_1 = require("./internal/message");
const request_1 = require("./internal/request");
const util_1 = require("./internal/util");
const websocket_1 = require("./internal/websocket");
class WebsocketClient {
    constructor(config = {}) {
        this.config = config;
        // currently joined-room, need cache to re-join on reconnect
        this.room = null;
        // wait request until connection opened
        this.sid = 0;
        this.initialize();
    }
    commit(params) {
        return this.send({
            method: 'commit',
            data: {
                kind: 'page',
                userId: params.userId,
                projectId: params.projectId,
                pageId: params.pageId,
                parentId: params.parentId,
                changes: request_1.createChanges(params.changes, params.userId),
                cursor: null,
                freeze: true,
            },
        });
    }
    join(params) {
        this.room = params;
        return this.send({
            method: 'room:join',
            data: {
                pageId: params.pageId,
                projectId: params.projectId,
                projectUpdatesStream: false,
            },
        });
    }
    initialize() {
        this.socket = this.openWebSocket();
        const { open, error, close, message } = websocket_1.getEventStream(this.socket);
        this.open$ = open.pipe(operators_1.shareReplay());
        this.error$ = error.pipe(operators_1.shareReplay());
        this.close$ = close.pipe(operators_1.shareReplay());
        this.message$ = message.pipe(operators_1.map((event) => message_1.parseMessage(event.data)));
        this.registerInitializationOnOpen();
        this.registerReconnectionOnClose();
    }
    async send(data) {
        const sid = String(this.sid++);
        const message = message_1.toSocketIoMessagePayload(sid, data);
        this.socket.send(message);
        return rxjs_1.firstValueFrom(this.message$.pipe(operators_1.first(message_1.isResponseMessageOf(sid)), operators_1.timeout(constants_1.CONFIG.responseTimeout)));
    }
    openWebSocket() {
        var _a, _b;
        const getSocket = (_a = this.config.websocketConstructor) !== null && _a !== void 0 ? _a : websocket_1.getWebSocket;
        return getSocket({ ...constants_1.CONFIG, cookie: util_1.getCookieToAuth((_b = this.config.token) !== null && _b !== void 0 ? _b : '') });
    }
    registerInitializationOnOpen() {
        this.message$.pipe(operators_1.first(message_1.isConnectionMessage)).subscribe(([, data]) => {
            this.socket.send(constants_1.packetTypes.connected);
            // start ping
            rxjs_1.interval(data.pingInterval)
                .pipe(operators_1.takeUntil(this.close$))
                .subscribe(() => this.socket.send(constants_1.packetTypes.ping));
            // for reconnection after closed
            if (this.room) {
                this.join(this.room);
            }
        });
    }
    registerReconnectionOnClose() {
        this.close$.subscribe(() => this.initialize());
    }
}
exports.WebsocketClient = WebsocketClient;
//# sourceMappingURL=websocket-client.js.map